import { Injectable, Inject, EventEmitter, Directive, ElementRef, Renderer2, Input, Output, Component, ContentChildren, NgModule } from '@angular/core';
import { annotate } from 'rough-notation';
import { Subject } from 'rxjs';
import { filter, scan, tap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

const defaultColors = {
    highlight: '#FFF176',
    circle: '#0D47A1',
    box: '#4A148C',
    'strike-through': '#1B5E1F',
    underline: '#B71C1B',
    'crossed-off': '#F57F17',
    bracket: '#FF0000',
};
const getDefaultTypeColor = (type) => {
    return defaultColors[type];
};
const defaultConfig = {
    type: 'highlight',
    color: getDefaultTypeColor('highlight'),
};

class RoughNotationService {
    constructor(defaultConfig) {
        this.defaultConfig = defaultConfig;
        this.visibilitySubject = new Subject();
        this.inputConfig = {};
        this.visibilityState$ = this.visibilitySubject.asObservable().pipe(filter(() => !!this.instance), scan((currentState) => !currentState, false), tap((show) => (show ? this.instance.show() : this.instance.hide())));
    }
    /**
     * Generate config object from defaults and input overrides
     */
    get config() {
        if (this.inputConfig.type && !this.inputConfig.color) {
            // When a type is specified without a color
            // Apply the default
            this.inputConfig.color = getDefaultTypeColor(this.inputConfig.type);
        }
        return Object.assign(Object.assign({}, this.defaultConfig), this.inputConfig);
    }
    instantiate(element) {
        this.instance = annotate(element, this.config);
        return this.instance;
    }
    destroy() {
        this.instance.remove();
    }
    toggleVisibility() {
        this.visibilitySubject.next();
    }
}
RoughNotationService.decorators = [
    { type: Injectable }
];
RoughNotationService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['DEFAULT_CONFIG',] }] }
];

class RoughNotationDirective {
    constructor(hostElement, renderer, service) {
        this.hostElement = hostElement;
        this.renderer = renderer;
        this.service = service;
        this.isShowingChange = new EventEmitter();
        // Storing initial host element color
        this.initialTextColor = hostElement.nativeElement.style.color;
        this.showStateSubscription = this.service.visibilityState$.subscribe((state) => {
            this.isShowingChange.emit(state);
            this._show = state;
            if (this.annotatedTextColor) {
                // In case the user specified an annotation text color
                // We set or reset it according to the state
                const newColor = state ? this.annotatedTextColor : this.initialTextColor;
                this.renderer.setStyle(this.hostElement.nativeElement, 'color', newColor);
            }
        });
    }
    set roughNotation(value) {
        if (typeof value === 'string') {
            if (value) {
                throw new Error('RoughAnnotationConfig object is expected as input');
            }
            this.service.inputConfig = {};
        }
        else {
            this.service.inputConfig = value;
        }
    }
    set show(value) {
        if (value === false || this.instance) {
            this.setVisibility(value);
        }
    }
    get show() {
        return this._show;
    }
    get instance() {
        return this.service.instance;
    }
    ngOnInit() {
        // Instantiate RoughNotation
        this.service.instantiate(this.hostElement.nativeElement);
        if (this._show !== false) {
            const animationDelay = this.service.config.animationDelay;
            if (animationDelay) {
                // In case an animationDelay is specified
                setTimeout(() => {
                    this.setVisibility(true);
                }, animationDelay);
            }
            else {
                this.setVisibility(true);
            }
        }
    }
    setVisibility(show) {
        if (this._show !== show) {
            this.toggle();
            this._show = show;
        }
    }
    toggle() {
        this.service.toggleVisibility();
    }
    ngOnDestroy() {
        var _a;
        (_a = this.showStateSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this.service.destroy();
    }
}
RoughNotationDirective.decorators = [
    { type: Directive, args: [{
                selector: '[roughNotation]',
                providers: [RoughNotationService],
                exportAs: 'roughNotation',
            },] }
];
RoughNotationDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: RoughNotationService }
];
RoughNotationDirective.propDecorators = {
    roughNotation: [{ type: Input, args: ['roughNotation',] }],
    show: [{ type: Input }],
    instance: [{ type: Input, args: ['instance',] }],
    annotatedTextColor: [{ type: Input }],
    isShowingChange: [{ type: Output }]
};

class RoughNotationGroupComponent {
    constructor() {
        this._show = true;
    }
    set show(value) {
        this.setVisibility(value);
    }
    ngAfterContentInit() {
        this.setVisibility(this._show);
    }
    setVisibility(show) {
        if (this._show !== show) {
            this.toggleGroupMembers();
            this._show = show;
        }
    }
    toggleGroupMembers() {
        this.annotations.forEach((annotation) => annotation.toggle());
    }
}
RoughNotationGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'rough-notation-group',
                template: "<ng-content></ng-content>\n",
                styles: [""]
            },] }
];
RoughNotationGroupComponent.propDecorators = {
    show: [{ type: Input }],
    annotations: [{ type: ContentChildren, args: [RoughNotationDirective, { descendants: true },] }]
};

const ɵ0 = defaultConfig;
class RoughNotationModule {
    static forRoot(defaultConfigOverride) {
        return {
            ngModule: RoughNotationModule,
            providers: [
                RoughNotationService,
                {
                    provide: 'DEFAULT_CONFIG',
                    useValue: defaultConfigOverride !== null && defaultConfigOverride !== void 0 ? defaultConfigOverride : defaultConfig,
                },
            ],
        };
    }
}
RoughNotationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RoughNotationDirective, RoughNotationGroupComponent],
                imports: [CommonModule],
                exports: [RoughNotationDirective, RoughNotationGroupComponent],
                providers: [
                    RoughNotationService,
                    {
                        provide: 'DEFAULT_CONFIG',
                        useValue: ɵ0,
                    },
                ],
            },] }
];

/*
 * Public API Surface of ng-rough-notation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { RoughNotationDirective, RoughNotationGroupComponent, RoughNotationModule, RoughNotationService, ɵ0, getDefaultTypeColor as ɵb, defaultConfig as ɵc };
//# sourceMappingURL=ng-rough-notation.js.map
